/*  LoRa Receiver Trasformatorio 2018
 *  Hardware: Adafruit Feather 32u4 with LoRa Radio, Sparkfun ESP8266 Thing Dev, DISPLAY OLED 128 x 32
 *  
 * This Sketch use Feather board as a radio, receiving packets from other Feather boards used as transmitters
 * This board re-transmit data to a Sparkfun ESP8266 or Arduino Ethernet via Software Serial, only if ID of transmitter is allowed
 * ID list is into TX_data.h  folder
 *  
 *  General Packet Structure

radiopacket[0]  = highByte(ID_TX);      Transmitting node identifier ID_TX
radiopacket[1]  = lowByte(ID_TX);    
radiopacket[2]  = highByte(Length);     Packet payload length
radiopacket[3]  = lowByte(Length);
radiopacket[4]  = highByte(vbatTX);     Power supply of the transmitting node in mV 
radiopacket[5]  = lowByte(vbatTX);
radiopacket[6]  = highByte(npacket);    Packet progressive number
radiopacket[7]  = lowByte(npacket);
radiopacket[8] =  highByte(Variable1);  Variable 1 value
radiopacket[9] =  lowByte(Variable1);      

....

    Upload with USB cable
   Setting the IDE, select "verbouse"
   Dopo che si clickato sul pulsante di download sulla IDE, aspettare che compaiano
   le scritte di ricerca della COM, a questo punto premere il bottone del reset.
   Se non funziona così, caricare prima uno sketch vuoto e rifare da capo

  Wiring 
*/
#include <SPI.h>
#include <RH_RF95.h>
#include <SoftwareSerial.h>
#include <Adafruit_SSD1306.h>  // OLED

 SoftwareSerial mySerial1(5, 6); //RX, TX   verso ESP8266 o ARduino Ethernet (6,7)

/* for feather32u4 */
#define RFM95_CS 8
#define RFM95_RST 4
#define RFM95_INT 7
#define RF95_FREQ 868.0
// Singleton instance of the radio driver
RH_RF95 rf95(RFM95_CS, RFM95_INT);

/*  for OLED */
#define OLED_RESET 11  // e' stato cambiato da 4 a 11 altrimenti fa casino con RFM95
Adafruit_SSD1306 display(OLED_RESET);

// Blinky on receipt
#define LED_pin 13
#include "TX_data.h"

//***************************************************************************
byte ECHO = 1;  // When 1 allows diagnostic 
                // Put 0 when the board is not connected to the USB cable.
//***************************************************************************
////////////////////FILTRO STAZIONI/////////////////////////////////
//word IDF1 = ID10;   // Basetta n.1
//word IDF2 = ID14;

//word IDF1 = ID11;   // Basetta n.2
//word IDF2 = ID15;

//word IDF1 = ID12;   // Basetta n.3
//word IDF2 = ID16;

//word IDF1 = ID13;   // Basetta n.4
//word IDF2 = ID17;
/////////////////////////////////////////////////////////////////////

byte  buf[RH_RF95_MAX_MESSAGE_LEN];  //  max 251 byte
byte  i = 0;
byte  len = 0;  
word  ID_TX = 0;
word  vBatTX = 0;
word  Length = 0;
word  npacket = 0;
float nRSSI=0;
word  wRSSI=0;
byte  prova = 0;
////////////////////////////////////////////////////////
void setup() 
{
  pinMode(5, INPUT);
  pinMode(6, OUTPUT);
  pinMode(LED_pin, OUTPUT);     
  pinMode(RFM95_RST, OUTPUT);
  digitalWrite(RFM95_RST, HIGH);

  // by default, we'll generate the high voltage from the 3.3v line internally! (neat!)
  display.begin(SSD1306_SWITCHCAPVCC, 0x3C);  // initialize with the I2C addr 0x3C (for the 128x32)
  // Clear the buffer.
  display.clearDisplay();

if(ECHO){
  while (!Serial);  
  Serial.begin(57600);
  delay(1000);
  Serial.println("feather_RX_ESP_OLED.ino");
}
  // manual reset
  digitalWrite(RFM95_RST, LOW);
  delay(10);
  digitalWrite(RFM95_RST, HIGH);
  delay(10);
  
// don't use lower speed 
//  mySerial1  to ESP or Ethernet
  mySerial1.begin(57600); 
  delay(1000);
  
// radio initialization
  while (!rf95.init()) {
    if(ECHO){
    Serial.println("LoRa radio init failed");
    }
    while (1);
  }
  
    if(ECHO){
      Serial.println("LoRa radio init OK!");
    }

  // Defaults after init are 434.0MHz, modulation GFSK_Rb250Fd250, +13dbM
  if (!rf95.setFrequency(RF95_FREQ)) {
    if(ECHO){
    Serial.println("setFrequency failed");
    }
    while (1);
  }
  if(ECHO){
     Serial.print("Set Freq to: "); Serial.println(RF95_FREQ);
  }
  
// Defaults after init are 434.0MHz, 13dBm, Bw = 125 kHz, Cr = 4/5, Sf = 128chips/symbol, CRC on
// The default transmitter power is 13dBm, using PA_BOOST.
// If you are using RFM95/96/97/98 modules which uses the PA_BOOST transmitter pin, then 
// you can set transmitter powers from 5 to 23 dBm:

/*  
 *   rf95.setTxPower(23, false); 
 *   rf95.setModemConfig(rf95.Bw31_25Cr48Sf512);
 *   rf95.setModemConfig(rf95.Bw125Cr45Sf128);
 *   rf95.setModemConfig(rf95.Bw125Cr48Sf4096);
 
Bw125Cr45Sf128   Bw = 125 kHz, Cr = 4/5, Sf = 128chips/symbol, CRC on. Default medium range. 
Bw500Cr45Sf128  Bw = 500 kHz, Cr = 4/5, Sf = 128chips/symbol, CRC on. Fast+short range. 
Bw31_25Cr48Sf512  Bw = 31.25 kHz, Cr = 4/8, Sf = 512chips/symbol, CRC on. Slow+long range. 
Bw125Cr48Sf4096   Bw = 125 kHz, Cr = 4/8, Sf = 4096chips/symbol, CRC on. Slow+long range. 
*/
/////////////////////////////////init Display OLED////////////////////////////////////


  if(ECHO){
     Serial.println(" Ho finito il setup");
  }
}
///////////////////////////////////////////////////////
void loop(){
  
    display.clearDisplay();
//////////////////////////////////simulazione///////////////////7

if(prova){
     ID_TX = 48001;
     vBatTX = 4900;
     Length = 10;
     npacket = 1567;
     word pesog = 1234;
     buf[0]  = highByte(ID_TX);  //    Transmitting node identifier ID_TX
     buf[1]  = lowByte(ID_TX);    
     buf[2]  = highByte(Length);   //  Packet payload length
     buf[3]  = lowByte(Length);
     buf[4]  = highByte(vBatTX);   //  Power supply of the transmitting node in mV 
     buf[5]  = lowByte(vBatTX);
     buf[6]  = highByte(npacket);   // Packet progressive number
     buf[7]  = lowByte(npacket);
     buf[8] =  highByte(pesog); // Variable 1 value
     buf[9] =  lowByte(pesog);    
     wRSSI=45;
     Display_OLED2();
     TrasmettiSeriale();
     delay(4000);
  }
  else{
    
        for(i=0; i < RH_RF95_MAX_MESSAGE_LEN; i++){
             buf[i] = 0;
        } 
        
        if (rf95.available()){
             lampeggia_2();
             len = sizeof(buf);
             if (rf95.recv(buf, &len)){
      
             if(ECHO){
                 Serial.println();
                 Serial.println();
//                 RH_RF95::printBuffer("Received: ", buf, len);
             }
      
              ID_TX =   word(buf[0],buf[1]);
              Length =  word(buf[2],buf[3]);
              vBatTX =  word(buf[4],buf[5]);
              npacket = word(buf[6],buf[7]);
              nRSSI =   rf95.lastRssi();
              wRSSI = word(-1*nRSSI);

              if(ECHO){
                  StampaRicezione();
              }


              if(len == Length){

                  TrasmettiSeriale();
                  if(ID_TX > 49000){
                     Display_OLED1();  // vibrazioni
                  }
                  if(ID_TX <= 48004){
                  Display_OLED2();  // bilancia
                  }                 
             }else{
                  if(ECHO){
                     Serial.println(" Lunghezza non coerente ");
                  }
             }         
       }else{   
           if(ECHO){
                Serial.println("Receive failed");
           }         
        }  
      }
  }
}


